@@ -4,7 +4,8 @@ if ( function_exists( 'wfLoadExtension' ) ) {
 	// Keep i18n globals so mergeMessageFileList.php doesn't break
 	$wgMessagesDirs['SyntaxHighlight_GeSHi'] = __DIR__ . '/i18n';
 	/* wfWarn(
-		'Deprecated PHP entry point used for SyntaxHighlight_GeSHi extension. Please use wfLoadExtension instead, ' .
+		'Deprecated PHP entry point used for SyntaxHighlight_GeSHi extension. '
+		'Please use wfLoadExtension instead, ' .
 		'see https://www.mediawiki.org/wiki/Extension_registration for more details.'
 	); */
 	return;
