@@ -204,7 +204,8 @@ class ChangesList extends ContextSource {
 		$code = $lang->getCode();
 		static $fastCharDiff = array();
 		if ( !isset( $fastCharDiff[$code] ) ) {
-			$fastCharDiff[$code] = $config->get( 'MiserMode' ) || $context->msg( 'rc-change-size' )->plain() === '$1';
+			$fastCharDiff[$code] = $config->get( 'MiserMode' )
+				|| $context->msg( 'rc-change-size' )->plain() === '$1';
 		}
 
 		$formattedSize = $lang->formatNum( $szdiff );
@@ -370,6 +371,19 @@ class ChangesList extends ContextSource {
 		$s .= " $articlelink";
 	}
 
+	/**
+	 * @param RecentChange $rc
+	 * @param bool $unpatrolled
+	 * @param bool $watched
+	 * @return string
+	 * @since 1.26
+	 */
+	public function getArticleLink( RecentChange $rc, $unpatrolled, $watched ) {
+		$s = '';
+		$this->insertArticleLink( $s, $rc, $unpatrolled, $watched );
+		return $s;
+	}
+
 	/**
 	 * Get the timestamp from $rc formatted with current user's settings
 	 * and a separator
@@ -542,6 +556,17 @@ class ChangesList extends ContextSource {
 		}
 	}
 
+	/**
+	 * @param RecentChange $rc
+	 * @return string
+	 * @since 1.26
+	 */
+	public function getRollback( RecentChange $rc ) {
+		$s = '';
+		$this->insertRollback( $s, $rc );
+		return $s;
+	}
+
 	/**
 	 * @param string $s
 	 * @param RecentChange $rc
@@ -560,6 +585,18 @@ class ChangesList extends ContextSource {
 		$s .= ' ' . $tagSummary;
 	}
 
+	/**
+	 * @param RecentChange $rc
+	 * @param array $classes
+	 * @return string
+	 * @since 1.26
+	 */
+	public function getTags( RecentChange $rc, array &$classes ) {
+		$s = '';
+		$this->insertTags( $s, $rc, $classes );
+		return $s;
+	}
+
 	public function insertExtra( &$s, &$rc, &$classes ) {
 		// Empty, used for subclasses to add anything special.
 	}
