@@ -193,6 +193,8 @@ class LogFormatter {
 	 * @return string Text
 	 */
 	public function getIRCActionText() {
+		global $wgContLang;
+
 		$this->plaintext = true;
 		$this->irctext = true;
 
@@ -271,6 +273,10 @@ class LogFormatter {
 						$text = wfMessage( 'modifiedarticleprotection' )
 							->rawParams( $target . ' ' . $parameters[0] )->inContentLanguage()->escaped();
 						break;
+					case 'move_prot':
+						$text = wfMessage( 'movedarticleprotection' )
+							->rawParams( $target, $parameters['4::oldtitle'] )->inContentLanguage()->escaped();
+						break;
 				}
 				break;
 
@@ -338,7 +344,6 @@ class LogFormatter {
 			case 'block':
 				switch ( $entry->getSubtype() ) {
 					case 'block':
-						global $wgContLang;
 						// Keep compatibility with extensions by checking for
 						// new key (5::duration/6::flags) or old key (0/optional 1)
 						if ( $entry->isLegacy() ) {
@@ -358,7 +363,6 @@ class LogFormatter {
 							->rawParams( $target )->inContentLanguage()->escaped();
 						break;
 					case 'reblock':
-						global $wgContLang;
 						$duration = $wgContLang->translateBlockExpiry( $parameters['5::duration'] );
 						$flags = BlockLogFormatter::formatBlockFlags( $parameters['6::flags'], $wgContLang );
 						$text = wfMessage( 'reblock-logentry' )
@@ -790,7 +794,7 @@ class LogFormatter {
 				break;
 
 			case 'number':
-				if ( ctype_digit( $value ) ) {
+				if ( ctype_digit( $value ) || is_int( $value ) ) {
 					$value = (int)$value;
 				} else {
 					$value = (float)$value;
