@@ -25,6 +25,7 @@
  * Let users reset tokens like the watchlist token.
  *
  * @ingroup SpecialPage
+ * @deprecated 1.26
  */
 class SpecialResetTokens extends FormSpecialPage {
 	private $tokensList;
@@ -123,6 +124,10 @@ class SpecialResetTokens extends FormSpecialPage {
 		}
 	}
 
+	protected function getDisplayFormat() {
+		return 'ooui';
+	}
+
 	public function onSubmit( array $formData ) {
 		if ( $formData['tokens'] ) {
 			$user = $this->getUser();
