@@ -126,7 +126,6 @@ $specialPageAliases = array(
 	'Newpages'                  => array( 'Керла_агӀонаш' ),
 	'PasswordReset'             => array( 'Пароль_кхоссар' ),
 	'PermanentLink'             => array( 'Гуттарлера_хьажорг' ),
-
 	'Preferences'               => array( 'ГӀирсаш' ),
 	'Prefixindex'               => array( 'Хьалха_агӀонийн_цӀераш_хӀотто_еза' ),
 	'Protectedpages'            => array( 'ГӀаролла_дина_агӀонаш' ),
@@ -311,3 +310,20 @@ $magicWords = array(
 	'url_wiki'                  => array( '0', 'ЧЕХКА', 'ВИКИ', 'WIKI' ),
 );
 
+$defaultDateFormat = 'ydm';
+
+$datePreferences = array(
+	'default',
+	'ydm',
+	'ISO 8601',
+);
+
+$dateFormats = array(
+	'ydm time' => 'H:i',
+	'ydm date' => 'Y, j F',
+	'ydm both' => 'Y, j F, H:i',
+
+	'ISO 8601 time' => 'xnH:xni:xns',
+	'ISO 8601 date' => 'xnY-xnm-xnd',
+	'ISO 8601 both' => 'xnY-xnm-xnd"T"xnH:xni:xns',
+);
