@@ -49,10 +49,10 @@ $namespaceAliases = array(
 	"討論" 			=> NS_TALK,
 	"讨论" 			=> NS_TALK,
 	"用户" 			=> NS_USER,
-	"用戶 對話" 		=> NS_USER_TALK,
-	"用户 对话" 		=> NS_USER_TALK,
-	"用戶 討論" 		=> NS_USER_TALK,
-	"用户 讨论" 		=> NS_USER_TALK,
+	"用戶_對話" 		=> NS_USER_TALK,
+	"用户_对话" 		=> NS_USER_TALK,
+	"用戶_討論" 		=> NS_USER_TALK,
+	"用户_讨论" 		=> NS_USER_TALK,
 	'$1_傾偈'		=> NS_PROJECT_TALK,
 	"檔" 			=> NS_FILE,
 	"檔案" 			=> NS_FILE,
@@ -64,35 +64,35 @@ $namespaceAliases = array(
 	"图像" 			=> NS_FILE,
 	'Image'                 => NS_FILE,
 	'Image_talk'            => NS_FILE_TALK,
-	"檔 討論" 		=> NS_FILE_TALK,
-	"档 讨论" 		=> NS_FILE_TALK,
-	"檔案 討論" 		=> NS_FILE_TALK,
-	"档案 讨论" 		=> NS_FILE_TALK,
-	"圖 討論" 		=> NS_FILE_TALK,
-	"图 讨论" 		=> NS_FILE_TALK,
-	"圖像 討論" 		=> NS_FILE_TALK,
-	"图像 讨论" 		=> NS_FILE_TALK,
+	"檔_討論" 		=> NS_FILE_TALK,
+	"档_讨论" 		=> NS_FILE_TALK,
+	"檔案_討論" 		=> NS_FILE_TALK,
+	"档案_讨论" 		=> NS_FILE_TALK,
+	"圖_討論" 		=> NS_FILE_TALK,
+	"图_讨论" 		=> NS_FILE_TALK,
+	"圖像_討論" 		=> NS_FILE_TALK,
+	"图像_讨论" 		=> NS_FILE_TALK,
 	'MediaWiki_傾偈'	=> NS_FILE_TALK,
-	"模 討論" 		=> NS_TEMPLATE_TALK,
-	"模 讨论" 		=> NS_TEMPLATE_TALK,
+	"模_討論" 		=> NS_TEMPLATE_TALK,
+	"模_讨论" 		=> NS_TEMPLATE_TALK,
 	"幫助" 			=> NS_HELP,
 	"說明" 			=> NS_HELP,
 	"帮手" 			=> NS_HELP,
 	"帮助" 			=> NS_HELP,
 	"说明" 			=> NS_HELP,
-	"幫手 討論" 		=> NS_HELP_TALK,
-	"幫助 討論" 		=> NS_HELP_TALK,
-	"說明 討論" 		=> NS_HELP_TALK,
-	"帮手 讨论" 		=> NS_HELP_TALK,
-	"帮助 讨论" 		=> NS_HELP_TALK,
-	"说明 讨论" 		=> NS_HELP_TALK,
+	"幫手_討論" 		=> NS_HELP_TALK,
+	"幫助_討論" 		=> NS_HELP_TALK,
+	"說明_討論" 		=> NS_HELP_TALK,
+	"帮手_讨论" 		=> NS_HELP_TALK,
+	"帮助_讨论" 		=> NS_HELP_TALK,
+	"说明_讨论" 		=> NS_HELP_TALK,
 	"類" 			=> NS_CATEGORY,
 	"类" 			=> NS_CATEGORY,
 	"分类" 			=> NS_CATEGORY,
-	"類 討論" 		=> NS_CATEGORY_TALK,
-	"分類 討論" 		=> NS_CATEGORY_TALK,
-	"类 讨论" 		=> NS_CATEGORY_TALK,
-	"分类 讨论" 		=> NS_CATEGORY_TALK,
+	"類_討論" 		=> NS_CATEGORY_TALK,
+	"分類_討論" 		=> NS_CATEGORY_TALK,
+	"类_讨论" 		=> NS_CATEGORY_TALK,
+	"分类_讨论" 		=> NS_CATEGORY_TALK,
 );
 
 $specialPageAliases = array(
@@ -151,7 +151,6 @@ $specialPageAliases = array(
 	'Newpages'                  => array( '新版' ),
 	'PasswordReset'             => array( '重設密碼' ),
 	'PermanentLink'             => array( '永久鏈' ),
-
 	'Preferences'               => array( '喜好設定' ),
 	'Prefixindex'               => array( '全部頁嘅前綴' ),
 	'Protectedpages'            => array( '保護頁' ),
