@@ -39,6 +39,7 @@ return array(
 		'group' => 'noscript',
 	),
 	'filepage' => array(
+		'position' => 'top',
 		'class' => 'ResourceLoaderWikiModule',
 		'styles' => array( 'MediaWiki:Filepage.css' ),
 	),
@@ -90,19 +91,22 @@ return array(
 	 * including more than one of them into your skin as this will result in duplicate CSS.
 	 */
 	'mediawiki.skinning.elements' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.skinning/elements.css' => array( 'media' => 'screen' ),
 		),
 	),
 	'mediawiki.skinning.content' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.skinning/elements.css' => array( 'media' => 'screen' ),
 			'resources/src/mediawiki.skinning/content.css' => array( 'media' => 'screen' ),
 		),
 	),
+	// Used in the web installer. Test it after modifying this definition!
 	'mediawiki.skinning.interface' => array(
+		'position' => 'top',
 		'class' => 'ResourceLoaderSkinModule',
-		// Used in the web installer. Test it after modifying this definition!
 		'styles' => array(
 			'resources/src/mediawiki.skinning/elements.css' => array( 'media' => 'screen' ),
 			'resources/src/mediawiki.skinning/content.css' => array( 'media' => 'screen' ),
@@ -111,6 +115,7 @@ return array(
 	),
 
 	'mediawiki.skinning.content.parsoid' => array(
+		'position' => 'top',
 		// Style Parsoid HTML+RDFa output consistent with wikitext from PHP parser
 		// with the interface.css styles; skinStyles should be used if your
 		// skin over-rides common content styling.
@@ -121,6 +126,7 @@ return array(
 	),
 
 	'mediawiki.skinning.content.externallinks' => array(
+		'position' => 'bottom',
 		'styles' => array(
 			'resources/src/mediawiki.skinning/content.externallinks.css' => array( 'media' => 'screen' ),
 		),
@@ -142,7 +148,7 @@ return array(
 		'scripts' => 'resources/src/jquery/jquery.accessKeyLabel.js',
 		'dependencies' => array(
 			'jquery.client',
-			'jquery.mwExtension',
+			'mediawiki.RegExp',
 		),
 		'messages' => array( 'brackets', 'word-separator' ),
 		'targets' => array( 'mobile', 'desktop' ),
@@ -209,7 +215,6 @@ return array(
 		'styles' => 'resources/src/jquery/jquery.confirmable.css',
 		'dependencies' => 'mediawiki.jqueryMsg',
 	),
-	// Use mediawiki.cookie in new code, rather than jquery.cookie.
 	'jquery.cookie' => array(
 		'scripts' => 'resources/lib/jquery/jquery.cookie.js',
 		'targets' => array( 'desktop', 'mobile' ),
@@ -242,12 +247,44 @@ return array(
 	),
 	'jquery.highlightText' => array(
 		'scripts' => 'resources/src/jquery/jquery.highlightText.js',
-		'dependencies' => 'jquery.mwExtension',
+		'dependencies' => array(
+			'mediawiki.RegExp',
+			'dom-level2-shim',
+		),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'jquery.hoverIntent' => array(
 		'scripts' => 'resources/lib/jquery/jquery.hoverIntent.js',
 	),
+	'jquery.i18n' => array(
+		'scripts' => array(
+			'resources/lib/jquery.i18n/src/jquery.i18n.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.messagestore.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.parser.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.emitter.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.emitter.bidi.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.language.js',
+			'resources/lib/jquery.i18n/src/jquery.i18n.fallbacks.js',
+		),
+		'dependencies' => 'mediawiki.libs.pluralruleparser',
+		'languageScripts' => array(
+			'bs' => 'resources/lib/jquery.i18n/src/languages/bs.js',
+			'dsb' => 'resources/lib/jquery.i18n/src/languages/dsb.js',
+			'fi' => 'resources/lib/jquery.i18n/src/languages/fi.js',
+			'ga' => 'resources/lib/jquery.i18n/src/languages/ga.js',
+			'he' => 'resources/lib/jquery.i18n/src/languages/he.js',
+			'hsb' => 'resources/lib/jquery.i18n/src/languages/hsb.js',
+			'hu' => 'resources/lib/jquery.i18n/src/languages/hu.js',
+			'hy' => 'resources/lib/jquery.i18n/src/languages/hy.js',
+			'la' => 'resources/lib/jquery.i18n/src/languages/la.js',
+			'ml' => 'resources/lib/jquery.i18n/src/languages/ml.js',
+			'os' => 'resources/lib/jquery.i18n/src/languages/os.js',
+			'ru' => 'resources/lib/jquery.i18n/src/languages/ru.js',
+			'sl' => 'resources/lib/jquery.i18n/src/languages/sl.js',
+			'uk' => 'resources/lib/jquery.i18n/src/languages/uk.js',
+		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 	'jquery.localize' => array(
 		'scripts' => 'resources/src/jquery/jquery.localize.js',
 	),
@@ -304,13 +341,15 @@ return array(
 		'styles' => 'resources/src/jquery/jquery.tablesorter.css',
 		'messages' => array( 'sort-descending', 'sort-ascending' ),
 		'dependencies' => array(
-			'jquery.mwExtension',
+			'dom-level2-shim',
+			'mediawiki.RegExp',
 			'mediawiki.language.months',
 		),
 	),
 	'jquery.textSelection' => array(
 		'scripts' => 'resources/src/jquery/jquery.textSelection.js',
 		'dependencies' => 'jquery.client',
+		'targets' => array( 'mobile', 'desktop' ),
 	),
 	'jquery.throttle-debounce' => array(
 		'scripts' => 'resources/lib/jquery/jquery.ba-throttle-debounce.js',
@@ -334,6 +373,13 @@ return array(
 
 	'jquery.ui.core' => array(
 		'scripts' => 'resources/lib/jquery.ui/jquery.ui.core.js',
+		'dependencies' => array(
+			'jquery.ui.core.styles',
+		),
+		'group' => 'jquery.ui',
+	),
+	'jquery.ui.core.styles' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/lib/jquery.ui/themes/smoothness/jquery.ui.core.css',
@@ -687,7 +733,10 @@ return array(
 	/* Moment.js */
 
 	'moment' => array(
-		'scripts' => 'resources/lib/moment/moment.js',
+		'scripts' => array(
+			'resources/lib/moment/moment.js',
+			'resources/src/moment-local-dmy.js',
+		),
 		'languageScripts' => array(
 			'af' => 'resources/lib/moment/locale/af.js',
 			'ar' => 'resources/lib/moment/locale/ar.js',
@@ -772,14 +821,14 @@ return array(
 	/* MediaWiki */
 
 	'mediawiki' => array(
-		// Keep maintenance/jsduck/eg-iframe.html in sync
+		'class' => 'ResourceLoaderRawFileModule',
+		// Keep in sync with maintenance/jsduck/eg-iframe.html
 		'scripts' => array(
+			'resources/lib/phpjs-sha1/sha1.js',
 			'resources/src/mediawiki/mediawiki.js',
 			'resources/src/mediawiki/mediawiki.errorLogger.js',
-			'resources/src/mediawiki/mediawiki.startUp.js',
 		),
 		'debugScripts' => 'resources/src/mediawiki/mediawiki.log.js',
-		'raw' => true,
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.apihelp' => array(
@@ -798,6 +847,12 @@ return array(
 			'resources/src/mediawiki/mediawiki.template.mustache.js',
 		),
 		'targets' => array( 'desktop', 'mobile' ),
+		'dependencies' => 'mediawiki.template',
+	),
+	'mediawiki.template.regexp' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.template.regexp.js',
+		'targets' => array( 'desktop', 'mobile' ),
+		'dependencies' => 'mediawiki.template',
 	),
 	'mediawiki.apipretty' => array(
 		'styles' => 'resources/src/mediawiki/mediawiki.apipretty.css',
@@ -806,7 +861,10 @@ return array(
 	),
 	'mediawiki.api' => array(
 		'scripts' => 'resources/src/mediawiki.api/mediawiki.api.js',
-		'dependencies' => 'mediawiki.util',
+		'dependencies' => array(
+			'mediawiki.util',
+			'user.tokens',
+		),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.api.category' => array(
@@ -821,7 +879,6 @@ return array(
 		'dependencies' => array(
 			'mediawiki.api',
 			'mediawiki.Title',
-			'user.tokens',
 		),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
@@ -837,15 +894,25 @@ return array(
 	'mediawiki.api.parse' => array(
 		'scripts' => 'resources/src/mediawiki.api/mediawiki.api.parse.js',
 		'dependencies' => 'mediawiki.api',
+		'targets' => array( 'desktop', 'mobile' ),
+	),
+	'mediawiki.api.upload' => array(
+		'scripts' => 'resources/src/mediawiki.api/mediawiki.api.upload.js',
+		'dependencies' => array(
+			'dom-level2-shim',
+			'mediawiki.api',
+			'mediawiki.api.edit',
+			'json',
+		),
 	),
 	'mediawiki.api.watch' => array(
 		'scripts' => 'resources/src/mediawiki.api/mediawiki.api.watch.js',
 		'dependencies' => array(
 			'mediawiki.api',
-			'user.tokens',
 		),
 	),
 	'mediawiki.content.json' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki/mediawiki.content.json.css',
 	),
 	'mediawiki.confirmCloseWindow' => array(
@@ -908,7 +975,10 @@ return array(
 			'feedback-useragent'
 		),
 	),
-
+	'mediawiki.feedlink' => array(
+		'position' => 'top',
+		'styles' => 'resources/src/mediawiki/mediawiki.feedlink.css',
+	),
 	'mediawiki.filewarning' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.filewarning.js',
 		'styles' => 'resources/src/mediawiki/mediawiki.filewarning.less',
@@ -916,8 +986,22 @@ return array(
 			'oojs-ui',
 		),
 	),
-
+	'mediawiki.ForeignApi' => array(
+		'targets' => array( 'desktop', 'mobile' ),
+		'class' => 'ResourceLoaderForeignApiModule',
+		// Additional dependencies generated dynamically
+		'dependencies' => 'mediawiki.ForeignApi.core',
+	),
+	'mediawiki.ForeignApi.core' => array(
+		'scripts' => 'resources/src/mediawiki.api/mediawiki.ForeignApi.js',
+		'dependencies' => array(
+			'mediawiki.api',
+			'oojs',
+		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 	'mediawiki.helplink' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki/mediawiki.helplink.less',
 		),
@@ -937,7 +1021,7 @@ return array(
 	'mediawiki.htmlform' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.htmlform.js',
 		'dependencies' => array(
-			'jquery.mwExtension',
+			'mediawiki.RegExp',
 			'jquery.byteLimit',
 		),
 		'messages' => array(
@@ -946,6 +1030,14 @@ return array(
 			'colon-separator',
 		),
 	),
+	'mediawiki.htmlform.styles' => array(
+		'styles' => 'resources/src/mediawiki/mediawiki.htmlform.css',
+		'position' => 'top',
+	),
+	'mediawiki.htmlform.ooui.styles' => array(
+		'styles' => 'resources/src/mediawiki/mediawiki.htmlform.ooui.css',
+		'position' => 'top',
+	),
 	'mediawiki.icon' => array(
 		'styles' => 'resources/src/mediawiki/mediawiki.icon.less',
 	),
@@ -953,6 +1045,7 @@ return array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.inspect.js',
 		'dependencies' => array(
 			'jquery.byteLength',
+			'mediawiki.RegExp',
 			'json',
 		),
 		'targets' => array( 'desktop', 'mobile' ),
@@ -980,10 +1073,13 @@ return array(
 	),
 	'mediawiki.notification' => array(
 		'styles' => array(
-			'resources/src/mediawiki/mediawiki.notification.css',
+			'resources/src/mediawiki/mediawiki.notification.common.css',
 			'resources/src/mediawiki/mediawiki.notification.hideForPrint.css'
 				=> array( 'media' => 'print' ),
 		),
+		'skinStyles' => array(
+			'default' => 'resources/src/mediawiki/mediawiki.notification.css',
+		),
 		'scripts' => 'resources/src/mediawiki/mediawiki.notification.js',
 		'dependencies' => 'mediawiki.page.startup',
 		'targets' => array( 'desktop', 'mobile' ),
@@ -992,6 +1088,10 @@ return array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.notify.js',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
+	'mediawiki.RegExp' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.RegExp.js',
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 	'mediawiki.pager.tablePager' => array(
 		'styles' => 'resources/src/mediawiki/mediawiki.pager.tablePager.less',
 		'position' => 'top',
@@ -1012,10 +1112,15 @@ return array(
 		),
 	),
 	'mediawiki.sectionAnchor' => array(
+		'position' => 'top',
 		// Back-compat to hide it on cached pages (T18691; Ie9e334e973; 2015-03-17)
 		'styles' => 'resources/src/mediawiki/mediawiki.sectionAnchor.css',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
+	'mediawiki.storage' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.storage.js',
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 	'mediawiki.Title' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.Title.js',
 		'dependencies' => array(
@@ -1024,21 +1129,157 @@ return array(
 		),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
+	'mediawiki.Upload' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.Upload.js',
+		'dependencies' => array(
+			'dom-level2-shim',
+			'mediawiki.api.upload',
+		),
+	),
+	'mediawiki.ForeignUpload' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.ForeignUpload.js',
+		'dependencies' => array(
+			'mediawiki.ForeignApi',
+			'mediawiki.Upload',
+			'oojs',
+		),
+	),
+	'mediawiki.ForeignStructuredUpload' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.ForeignStructuredUpload.js',
+		'dependencies' => array(
+			'mediawiki.ForeignUpload',
+		),
+	),
+	'mediawiki.Upload.Dialog' => array(
+		'scripts' => array(
+			'resources/src/mediawiki/mediawiki.Upload.Dialog.js',
+		),
+		'dependencies' => array(
+			'mediawiki.Upload.BookletLayout',
+		),
+		'messages' => array(
+			'upload-dialog-title',
+			'upload-dialog-button-cancel',
+			'upload-dialog-button-done',
+			'upload-dialog-button-save',
+			'upload-dialog-button-upload',
+		),
+	),
+	'mediawiki.Upload.BookletLayout' => array(
+		'scripts' => array(
+			'resources/src/mediawiki/mediawiki.Upload.BookletLayout.js',
+		),
+		'dependencies' => array(
+			'oojs-ui',
+			'mediawiki.Upload',
+			'mediawiki.jqueryMsg',
+		),
+		'messages' => array(
+			'upload-form-label-select-file',
+			'upload-form-label-infoform-title',
+			'upload-form-label-infoform-name',
+			'upload-form-label-infoform-description',
+			'upload-form-label-usage-title',
+			'upload-form-label-usage-filename',
+			'api-error-unknownerror',
+			'api-error-unknown-warning',
+			'api-error-badaccess-groups',
+			'api-error-badtoken',
+			'api-error-copyuploaddisabled',
+			'api-error-duplicate',
+			'api-error-duplicate-archive',
+			'api-error-empty-file',
+			'api-error-emptypage',
+			'api-error-fetchfileerror',
+			'api-error-fileexists-forbidden',
+			'api-error-fileexists-shared-forbidden',
+			'api-error-file-too-large',
+			'api-error-filename-tooshort',
+			'api-error-filetype-banned',
+			'api-error-filetype-banned-type',
+			'api-error-filetype-missing',
+			'api-error-hookaborted',
+			'api-error-http',
+			'api-error-illegal-filename',
+			'api-error-internal-error',
+			'api-error-invalid-file-key',
+			'api-error-missingparam',
+			'api-error-missingresult',
+			'api-error-mustbeloggedin',
+			'api-error-mustbeposted',
+			'api-error-noimageinfo',
+			'api-error-nomodule',
+			'api-error-ok-but-empty',
+			'api-error-overwrite',
+			'api-error-stashfailed',
+			'api-error-publishfailed',
+			'api-error-stasherror',
+			'api-error-stashedfilenotfound',
+			'api-error-stashpathinvalid',
+			'api-error-stashfilestorage',
+			'api-error-stashzerolength',
+			'api-error-stashnotloggedin',
+			'api-error-stashwrongowner',
+			'api-error-stashnosuchfilekey',
+			'api-error-timeout',
+			'api-error-unclassified',
+			'api-error-unknown-code',
+			'api-error-unknown-error',
+			'api-error-uploaddisabled',
+			'api-error-verification-error',
+			'fileexists',
+			'filepageexists',
+			'filename-bad-prefix',
+			'filename-thumb-name',
+			'badfilename',
+			'api-error-duplicate-archive',
+			'api-error-blacklisted', // HACK
+		),
+	),
+	'mediawiki.ForeignStructuredUpload.BookletLayout' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.ForeignStructuredUpload.BookletLayout.js',
+		'styles' => 'resources/src/mediawiki/mediawiki.ForeignStructuredUpload.BookletLayout.css',
+		'dependencies' => array(
+			'mediawiki.ForeignStructuredUpload',
+			'mediawiki.Upload.BookletLayout',
+			'mediawiki.widgets.CategorySelector',
+			'mediawiki.widgets.DateInputWidget',
+			'mediawiki.jqueryMsg',
+		),
+		'messages' => array(
+			'foreign-structured-upload-form-label-own-work',
+			'foreign-structured-upload-form-label-infoform-categories',
+			'foreign-structured-upload-form-label-infoform-date',
+			'foreign-structured-upload-form-label-own-work-message-default',
+			'foreign-structured-upload-form-label-not-own-work-message-default',
+			'foreign-structured-upload-form-label-not-own-work-local-default',
+			'foreign-structured-upload-form-label-own-work-message-shared',
+			'foreign-structured-upload-form-label-not-own-work-message-shared',
+			'foreign-structured-upload-form-label-not-own-work-local-shared',
+			'foreign-structured-upload-form-label-own-work-message-local',
+			'foreign-structured-upload-form-label-not-own-work-message-local',
+			'foreign-structured-upload-form-label-not-own-work-local-local',
+		),
+	),
 	'mediawiki.toc' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.toc.js',
-		'dependencies' => 'jquery.cookie',
+		'dependencies' => 'mediawiki.cookie',
 		'messages' => array( 'showtoc', 'hidetoc' ),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.Uri' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.Uri.js',
+		'templates' => array(
+			'strict.regexp' => 'resources/src/mediawiki/mediawiki.Uri.strict.regexp',
+			'loose.regexp' => 'resources/src/mediawiki/mediawiki.Uri.loose.regexp',
+		),
 		'dependencies' => 'mediawiki.util',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.user' => array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.user.js',
 		'dependencies' => array(
-			'jquery.cookie',
+			'mediawiki.cookie',
 			'mediawiki.api',
 			'user.options',
 			'user.tokens',
@@ -1056,7 +1297,7 @@ return array(
 		'scripts' => 'resources/src/mediawiki/mediawiki.util.js',
 		'dependencies' => array(
 			'jquery.accessKeyLabel',
-			'jquery.mwExtension',
+			'mediawiki.RegExp',
 			'mediawiki.notify',
 		),
 		'position' => 'top', // For $wgPreloadJavaScriptMwUtil
@@ -1073,6 +1314,10 @@ return array(
 		'styles' => 'resources/src/mediawiki.toolbar/toolbar.less',
 		'position' => 'top',
 	),
+	'mediawiki.experiments' => array(
+		'scripts' => 'resources/src/mediawiki/mediawiki.experiments.js',
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 
 	/* MediaWiki Action */
 
@@ -1087,15 +1332,15 @@ return array(
 		'position' => 'top',
 	),
 	'mediawiki.action.edit.styles' => array(
-		'styles' => 'resources/src/mediawiki.action/mediawiki.action.edit.styles.css',
 		'position' => 'top',
+		'styles' => 'resources/src/mediawiki.action/mediawiki.action.edit.styles.css',
 	),
 	'mediawiki.action.edit.collapsibleFooter' => array(
 		'scripts' => 'resources/src/mediawiki.action/mediawiki.action.edit.collapsibleFooter.js',
 		'styles' => 'resources/src/mediawiki.action/mediawiki.action.edit.collapsibleFooter.css',
 		'dependencies' => array(
 			'jquery.makeCollapsible',
-			'jquery.cookie',
+			'mediawiki.cookie',
 			'mediawiki.icon',
 		),
 	),
@@ -1134,18 +1379,18 @@ return array(
 		),
 	),
 	'mediawiki.action.history' => array(
+		'position' => 'top',
 		'scripts' => 'resources/src/mediawiki.action/mediawiki.action.history.js',
 		'styles' => 'resources/src/mediawiki.action/mediawiki.action.history.css',
-		'group' => 'mediawiki.action.history',
 	),
 	'mediawiki.action.history.diff' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.action/mediawiki.action.history.diff.css',
 			'resources/src/mediawiki.action/mediawiki.action.history.diff.print.css' => array(
 				'media' => 'print'
 			),
 		),
-		'group' => 'mediawiki.action.history',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.action.view.dblClickEdit' => array(
@@ -1165,6 +1410,7 @@ return array(
 		),
 	),
 	'mediawiki.action.view.categoryPage.styles' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki.action/mediawiki.action.view.categoryPage.less',
 		'targets' => array( 'desktop', 'mobile' )
 	),
@@ -1190,8 +1436,8 @@ return array(
 		'position' => 'top',
 	),
 	'mediawiki.action.view.redirectPage' => array(
-		'styles' => 'resources/src/mediawiki.action/mediawiki.action.view.redirectPage.css',
 		'position' => 'top',
+		'styles' => 'resources/src/mediawiki.action/mediawiki.action.view.redirectPage.css',
 	),
 	'mediawiki.action.view.rightClickEdit' => array(
 		'scripts' => 'resources/src/mediawiki.action/mediawiki.action.view.rightClickEdit.js',
@@ -1210,6 +1456,13 @@ return array(
 			'prefs-editing'
 		),
 	),
+	'mediawiki.action.view.filepage' => array(
+		'styles' => array(
+			'resources/src/mediawiki.action/mediawiki.action.view.filepage.print.css' => array( 'media' => 'print' ),
+			'resources/src/mediawiki.action/mediawiki.action.view.filepage.css',
+		),
+		'position' => 'top',
+	),
 
 	/* MediaWiki Language */
 
@@ -1265,10 +1518,14 @@ return array(
 	),
 
 	'mediawiki.jqueryMsg' => array(
+		// Add data for mediawiki.jqueryMsg, such as allowed tags
+		'class' => 'ResourceLoaderJqueryMsgModule',
 		'scripts' => 'resources/src/mediawiki/mediawiki.jqueryMsg.js',
 		'dependencies' => array(
 			'mediawiki.util',
 			'mediawiki.language',
+			'user.options',
+			'dom-level2-shim',
 		),
 		'targets' => array( 'desktop', 'mobile' ),
 	),
@@ -1300,9 +1557,18 @@ return array(
 	'mediawiki.page.gallery' => array(
 		'scripts' => 'resources/src/mediawiki.page/mediawiki.page.gallery.js',
 		'dependencies' => array(
+			'mediawiki.page.gallery.styles',
 			'jquery.throttle-debounce',
 		)
 	),
+	'mediawiki.page.gallery.styles' => array(
+		'styles' => array(
+			'resources/src/mediawiki.page/mediawiki.page.gallery.print.css' => array( 'media' => 'print' ),
+			'resources/src/mediawiki.page/mediawiki.page.gallery.css',
+		),
+		'position' => 'top',
+		'targets' => array( 'desktop', 'mobile' ),
+	),
 	'mediawiki.page.ready' => array(
 		'scripts' => 'resources/src/mediawiki.page/mediawiki.page.ready.js',
 		'dependencies' => array(
@@ -1345,7 +1611,7 @@ return array(
 			'mediawiki.page.startup',
 			'mediawiki.util',
 			'jquery.accessKeyLabel',
-			'jquery.mwExtension',
+			'mediawiki.RegExp',
 		),
 		'messages' => array(
 			'watch',
@@ -1369,6 +1635,7 @@ return array(
 	/* MediaWiki Special pages */
 
 	'mediawiki.special' => array(
+		'position' => 'top',
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.js',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.css',
 	),
@@ -1387,19 +1654,22 @@ return array(
 		),
 	),
 	'mediawiki.special.changeslist' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.changeslist.css',
 	),
 	'mediawiki.special.changeslist.legend' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.changeslist.legend.css',
 	),
 	'mediawiki.special.changeslist.legend.js' => array(
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.changeslist.legend.js',
 		'dependencies' => array(
 			'jquery.makeCollapsible',
-			'jquery.cookie',
+			'mediawiki.cookie',
 		),
 	),
 	'mediawiki.special.changeslist.enhanced' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.changeslist.enhanced.css',
 	),
 	'mediawiki.special.edittags' => array(
@@ -1421,12 +1691,20 @@ return array(
 	),
 	'mediawiki.special.movePage' => array(
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.movePage.js',
-		'dependencies' => 'jquery.byteLimit',
+		'dependencies' => array(
+			'jquery.byteLimit',
+			'mediawiki.widgets',
+		),
+	),
+	'mediawiki.special.movePage.styles' => array(
+		'styles' => 'resources/src/mediawiki.special/mediawiki.special.movePage.css',
+		'position' => 'top',
 	),
 	'mediawiki.special.pageLanguage' => array(
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.pageLanguage.js',
 	),
 	'mediawiki.special.pagesWithProp' => array(
+		'position' => 'top',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.pagesWithProp.css',
 	),
 	'mediawiki.special.preferences' => array(
@@ -1437,10 +1715,12 @@ return array(
 			'prefs-tabs-navigation-hint',
 			'prefswarning-warning',
 			'saveprefs',
+			'savedprefs',
 		),
 		'dependencies' => array(
 			'mediawiki.language',
 			'mediawiki.confirmCloseWindow',
+			'mediawiki.notification',
 		),
 	),
 	'mediawiki.special.recentchanges' => array(
@@ -1449,6 +1729,7 @@ return array(
 		'position' => 'top',
 	),
 	'mediawiki.special.search' => array(
+		'position' => 'top',
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.search.js',
 		'styles' => 'resources/src/mediawiki.special/mediawiki.special.search.css',
 		'messages' => array(
@@ -1494,25 +1775,16 @@ return array(
 		'position' => 'top',
 	),
 	'mediawiki.special.userlogin.signup.styles' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.special/mediawiki.special.userlogin.signup.css',
 		),
-		'position' => 'top',
 	),
 	'mediawiki.special.userlogin.login.styles' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.special/mediawiki.special.userlogin.login.css',
 		),
-		'position' => 'top',
-	),
-	'mediawiki.special.userlogin.common.js' => array(
-		'scripts' => array(
-			'resources/src/mediawiki.special/mediawiki.special.userlogin.common.js',
-		),
-		'messages' => array(
-			'createacct-captcha',
-			'createacct-imgcaptcha-ph',
-		),
 	),
 	'mediawiki.special.userlogin.signup.js' => array(
 		'scripts' => 'resources/src/mediawiki.special/mediawiki.special.userlogin.signup.js',
@@ -1564,8 +1836,8 @@ return array(
 
 	/* MediaWiki Installer */
 
+	// Used in the web installer. Test it after modifying this definition!
 	'mediawiki.legacy.config' => array(
-		// Used in the web installer. Test it after modifying this definition!
 		// These files are not actually loaded via ResourceLoader, so dependencies etc. won't work.
 		'scripts' => 'mw-config/config.js',
 		'styles' => 'mw-config/config.css',
@@ -1573,15 +1845,8 @@ return array(
 
 	/* MediaWiki Legacy */
 
-	'mediawiki.legacy.ajax' => array(
-		'scripts' => 'resources/src/mediawiki.legacy/ajax.js',
-		'dependencies' => array(
-			'mediawiki.util',
-			'mediawiki.legacy.wikibits',
-		),
-		'position' => 'top',
-	),
 	'mediawiki.legacy.commonPrint' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.legacy/commonPrint.css' => array( 'media' => 'print' )
 		),
@@ -1591,13 +1856,15 @@ return array(
 		'dependencies' => 'jquery.byteLimit',
 		'messages' => array( 'protect-unchain-permissions' )
 	),
+	// Used in the web installer. Test it after modifying this definition!
 	'mediawiki.legacy.shared' => array(
-		// Used in the web installer. Test it after modifying this definition!
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.legacy/shared.css' => array( 'media' => 'screen' )
 		),
 	),
 	'mediawiki.legacy.oldshared' => array(
+		'position' => 'top',
 		'styles' => array(
 			'resources/src/mediawiki.legacy/oldshared.css' => array( 'media' => 'screen' )
 		),
@@ -1606,85 +1873,175 @@ return array(
 		'scripts' => 'resources/src/mediawiki.legacy/wikibits.js',
 		'dependencies' => 'mediawiki.util',
 		'position' => 'top',
+		'targets' => array( 'desktop', 'mobile' ),
 	),
 
 	/* MediaWiki UI */
 
 	'mediawiki.ui' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/default.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.ui.checkbox' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/checkbox.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.ui.radio' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/radio.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	// Lightweight module for anchor styles
 	'mediawiki.ui.anchor' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/anchors.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	// Lightweight module for button styles
 	'mediawiki.ui.button' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/buttons.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.ui.input' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/inputs.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	'mediawiki.ui.icon' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/icons.less',
 			),
 		),
-		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
 	// Lightweight module for text styles
 	'mediawiki.ui.text' => array(
+		'position' => 'top',
 		'skinStyles' => array(
 			'default' => array(
 				'resources/src/mediawiki.ui/components/text.less',
 			),
 		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
+
+	'mediawiki.widgets' => array(
+		'scripts' => array(
+			'resources/src/mediawiki.widgets/mw.widgets.NamespaceInputWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.ComplexNamespaceInputWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.TitleInputWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.ComplexTitleInputWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.TitleOptionWidget.js',
+		),
+		'skinStyles' => array(
+			'default' => array(
+				'resources/src/mediawiki.widgets/mw.widgets.TitleInputWidget.css',
+			),
+		),
+		'dependencies' => array(
+			'oojs-ui',
+			'mediawiki.widgets.styles',
+			// TitleInputWidget
+			'mediawiki.Title',
+			'mediawiki.api',
+			'jquery.byteLimit',
+			// TitleOptionWidget
+			'jquery.autoEllipsis',
+			// CategorySelector
+			'mediawiki.ForeignApi',
+			// FIXME: Kept for bc
+			'mediawiki.widgets.CategorySelector',
+		),
+		'messages' => array(
+			// NamespaceInputWidget
+			'blanknamespace',
+			'namespacesall',
+			// TitleInputWidget
+			'mw-widgets-titleinput-description-new-page',
+			'mw-widgets-titleinput-description-redirect',
+		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
+	'mediawiki.widgets.styles' => array(
+		'skinStyles' => array(
+			'default' => array(
+				'resources/src/mediawiki.widgets/mw.widgets.ComplexNamespaceInputWidget.base.css',
+				'resources/src/mediawiki.widgets/mw.widgets.ComplexTitleInputWidget.base.css',
+			),
+		),
 		'position' => 'top',
 		'targets' => array( 'desktop', 'mobile' ),
 	),
+	'mediawiki.widgets.DateInputWidget' => array(
+		'scripts' => array(
+			'resources/src/mediawiki.widgets/mw.widgets.CalendarWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.DateInputWidget.js',
+		),
+		'skinStyles' => array(
+			'default' => array(
+				'resources/src/mediawiki.widgets/mw.widgets.CalendarWidget.less',
+				'resources/src/mediawiki.widgets/mw.widgets.DateInputWidget.less',
+			),
+		),
+		'messages' => array(
+			'mw-widgets-dateinput-no-date',
+			'mw-widgets-dateinput-placeholder-day',
+			'mw-widgets-dateinput-placeholder-month',
+		),
+		'dependencies' => array(
+			'oojs-ui',
+			'moment',
+		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
+	'mediawiki.widgets.CategorySelector' => array(
+		'scripts' => array(
+			'resources/src/mediawiki.widgets/mw.widgets.CategoryCapsuleItemWidget.js',
+			'resources/src/mediawiki.widgets/mw.widgets.CategorySelector.js',
+		),
+		'dependencies' => array(
+			'oojs-ui',
+			'mediawiki.api',
+		),
+		'targets' => array( 'desktop', 'mobile' ),
+	),
+	'mediawiki.widgets.UserInputWidget' => array(
+		'scripts' => array(
+			'resources/src/mediawiki.widgets/mw.widgets.UserInputWidget.js',
+		),
+		'dependencies' => array(
+			'oojs-ui',
+		),
+	),
 
 	/* es5-shim */
 	'es5-shim' => array(
@@ -1696,6 +2053,14 @@ return array(
 		'skipFunction' => 'resources/src/es5-skip.js',
 	),
 
+	/* dom-level2-shim */
+	// IE 8
+	'dom-level2-shim' => array(
+		'scripts' => 'resources/src/polyfill-nodeTypes.js',
+		'targets' => array( 'desktop', 'mobile' ),
+		'skipFunction' => 'resources/src/dom-level2-skip.js',
+	),
+
 	/* OOjs */
 	'oojs' => array(
 		'scripts' => array(
@@ -1711,41 +2076,5 @@ return array(
 	/* OOjs UI */
 	// WARNING: OOjs-UI is NOT TESTED with older browsers and is likely to break
 	// if loaded in browsers that don't support ES5
-	'oojs-ui' => array(
-		'scripts' => array(
-			'resources/lib/oojs-ui/oojs-ui.js',
-		),
-		'skinScripts' => array(
-			'default' => 'resources/lib/oojs-ui/oojs-ui-mediawiki.js',
-		),
-		'dependencies' => array(
-			'es5-shim',
-			'oojs',
-			'oojs-ui.styles',
-		),
-		'messages' => array(
-			'ooui-dialog-message-accept',
-			'ooui-dialog-message-reject',
-			'ooui-dialog-process-continue',
-			'ooui-dialog-process-dismiss',
-			'ooui-dialog-process-error',
-			'ooui-dialog-process-retry',
-			'ooui-outline-control-move-down',
-			'ooui-outline-control-move-up',
-			'ooui-outline-control-remove',
-			'ooui-toolbar-more',
-			'ooui-toolgroup-collapse',
-			'ooui-toolgroup-expand',
-		),
-		'targets' => array( 'desktop', 'mobile' ),
-	),
-
-	'oojs-ui.styles' => array(
-		'position' => 'top',
-		'skinStyles' => array(
-			'default' => 'resources/lib/oojs-ui/oojs-ui-mediawiki.css',
-		),
-		'targets' => array( 'desktop', 'mobile' ),
-	),
-
+	// @see ResourcesOOUI.php
 );
