@@ -159,8 +159,13 @@
 		}
 
 		if ( e ) {
-			if ( e.type === 'click' && options.linksPassthru && $.nodeName( e.target, 'a' ) ) {
-				// Don't fire if a link was clicked, if requested  (for premade togglers by default)
+			if (
+				e.type === 'click' &&
+				options.linksPassthru &&
+				$.nodeName( e.target, 'a' ) &&
+				$( e.target ).attr( 'href' ) !== '#'
+			) {
+				// Don't fire if a link with href !== '#' was clicked, if requested  (for premade togglers by default)
 				return;
 			} else if ( e.type === 'keypress' && e.which !== 13 && e.which !== 32 ) {
 				// Only handle keypresses on the "Enter" or "Space" keys
